//------------------------------------------------------------------------------
//TTexture class
//Written by Ananth B. AKA HellSpawn
//------------------------------------------------------------------------------
unit GLTexture;

interface

uses
    Windows,
    Classes,
    SysUtils,
    JPEG,
    Graphics,
    OpenGL,
    GL,
    GLU,
    //This file contains the OpenGL extension definitions
    GLExt;

const
     MINFILTER:GLenum=GL_LINEAR;
     MAGFILTER:GLenum=GL_LINEAR;

type
    //This will be the structure that will hold the basic image data that
    //we will give to OpenGL
    TTextureImage=record
                        ImageData:array of GLubyte;
                        BPP:GLuint;
                        Width,Height:GLuint;
                        TexID:GLuint;
    end;
    //This is the header for the uncompressed TGA that we will load
    TTGAHeader=array[0..11] of byte;
    //This is the header for the Quake2 WAL format.
    TWALHeader=record
                     Name:array[0..31] of char;
                     Width,Height:longword;
                     Offset:array[0..3] of longint;
                     NextName:array[0..31] of char;
                     Flags,
                     Contents,
                     Value:longword;
    end;
const
     TGAHeader:TTGAHeader=(0,0,2,0,0,0,0,0,0,0,0,0);
     MultiTextureAvailable:boolean=true;
     WAL_Brightness:integer=0;
type
    TTexture=class
                  private
                  protected
                  fTextureData:TTextureImage;
                  fMIPMapping:boolean;
                  fName:string;
                  function GetID:GLuint;
                  public
                  function LoadTexture(aFilename:string):boolean;
                  function LoadTGA(aFileName:string):boolean;
                  function LoadBitmap(aFilename:string):boolean;
                  function LoadWAL(aFilename,PaletteFilename:string):boolean;
                  function LoadJPEG(aFilename:string):Boolean;
                  procedure Enable;virtual;
                  procedure Disable;virtual;
                  procedure Transparency;
                  procedure Translucency;
                  procedure LightMap;
                  procedure Opaque;
                  destructor Destroy;override;
                  property ID:GLuint
                           read GetID;
                  property  Width:GLuint
                            read fTextureData.Width;
                  property Height:GLuint
                           read fTextureData.Height;
                  property MIPMapping:boolean
                           read fMIPMapping
                           write fMIPMapping;
                  property Name:string
                           read fName
                           write fName;
    end;

implementation
function TTexture.GetID:GLuint;
begin
     result:=fTextureData.TexID;
end;
procedure TTexture.Enable;
begin
     glBindTexture(GL_TEXTURE_2D,fTextureData.TexID);
end;
procedure TTexture.Disable;
begin
     glBindTexture(GL_TEXTURE_2D,0);
end;
procedure TTexture.Transparency;
begin
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
end;
procedure TTexture.Translucency;
begin
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE);
end;
procedure TTexture.LightMap;
begin
     glEnable(GL_BLEND);
     glBlendFunc(GL_DST_COLOR, GL_SRC_COLOR);
end;
procedure TTexture.Opaque;
begin
     glDisable(GL_BLEND);
end;
destructor TTexture.Destroy;
begin
     glDeleteTextures(1,@fTextureData.TexID);
     inherited;
end;

function TTexture.LoadTGA(aFileName:string):boolean;
var
   hdr:TTGAHeader;
   useful:array[0..5] of byte;
   BytesPerPixel,
   ImageSize,
   temp,
   imagetype:GLuint;
   f:file;
   i:GLuint;
begin
     result:=false;
     imagetype:=GL_RGBA;
     if not FileExists(aFileName) then exit;

     Assign(f,aFileName);
     try
        Reset(f,1);
        BlockRead(f,hdr,SizeOf(hdr));
        if not CompareMem(@hdr[0],@TGAHeader[0],SizeOf(hdr)) then exit;
        BlockRead(f,useful,SizeOf(useful));
     except
           exit;
     end;
     fTextureData.Width:=useful[1]*256 + useful[0];
     fTextureData.Height:=useful[3]*256 + useful[2];
     if (fTextureData.Width<=0)or
        (fTextureData.Height<=0) or
        ((useful[4]<>24) and (useful[4]<>32)) then
        exit;
     fTextureData.BPP:=useful[4];
     BytesPerPixel:=fTextureData.BPP div 8;
     ImageSize:=fTextureData.Width*fTextureData.Height*BytesPerPixel;
     try
        SetLength(fTextureData.ImageData,ImageSize);
     except
           exit;
     end;
     try
        BlockRead(f,fTextureData.ImageData[0],ImageSize)
     except
           fTextureData.ImageData:=nil;
           exit;
     end;
     i:=0;
     while i<ImageSize do
     begin
          temp:=fTextureData.Imagedata[i];
          fTextureData.Imagedata[i]:=fTextureData.Imagedata[i+2];
          fTextureData.Imagedata[i+2]:=temp;
          Inc(i,BytesPerPixel);
     end;
     Close(f);

     glGenTextures(1,@fTextureData.TexID);
     glBindTexture(GL_TEXTURE_2D, fTextureData.TexID);
     if not fMIPMapping then
     begin
          if fTextureData.BPP=24 then
             imagetype:=GL_RGB;
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,MINFILTER);
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,MAGFILTER);
          glTexImage2D(GL_TEXTURE_2D,0,imagetype,fTextureData.Width,fTextureData.Height,0,imagetype,GL_UNSIGNED_BYTE,@fTextureData.ImageData[0]);
     end
     else
     begin
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,MINFILTER);
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,MAGFILTER);
          if fTextureData.BPP=24 then
             imagetype:=GL_RGB;
          gluBuild2DMipmaps(GL_TEXTURE_2D, 3, fTextureData.Width, fTextureData.Height, imagetype ,GL_UNSIGNED_BYTE, @fTextureData.ImageData[0]);
     end;
     result:=true;
end;
function TTexture.LoadBitmap(aFilename:string):boolean;
var
   FileHeader: BITMAPFILEHEADER;
   InfoHeader: BITMAPINFOHEADER;
   Palette: array of RGBQUAD;
   BitmapFile: THandle;
   BitmapLength: LongWord;
   PaletteLength: LongWord;
   ReadBytes: LongWord;
   Front: ^Byte;
   Back: ^Byte;
   Temp: Byte;
   I : Integer;
   Width, Height : Integer;
   pData : Pointer;

begin
     result :=FALSE;
     BitmapFile := CreateFile(PChar(aFilename), GENERIC_READ, FILE_SHARE_READ, nil, OPEN_EXISTING, 0, 0);
     if (BitmapFile = INVALID_HANDLE_VALUE) then
        exit;

     // Get header information
     ReadFile(BitmapFile, FileHeader, SizeOf(FileHeader), ReadBytes, nil);
     ReadFile(BitmapFile, InfoHeader, SizeOf(InfoHeader), ReadBytes, nil);

     // Get palette
     PaletteLength := InfoHeader.biClrUsed;
     SetLength(Palette, PaletteLength);
     ReadFile(BitmapFile, Palette, PaletteLength, ReadBytes, nil);
     if (ReadBytes <> PaletteLength) then
        exit;

     Width  := InfoHeader.biWidth;
     Height := InfoHeader.biHeight;
     BitmapLength := InfoHeader.biSizeImage;
     if BitmapLength = 0 then
       BitmapLength := Width * Height * InfoHeader.biBitCount Div 8;

     // Get the actual pixel data
     GetMem(pData, BitmapLength);
     ReadFile(BitmapFile, pData^, BitmapLength, ReadBytes, nil);
     if (ReadBytes <> BitmapLength) then
        exit;

     CloseHandle(BitmapFile);

     // Bitmaps are stored BGR and not RGB, so swap the R and B bytes.
     for I :=0 to Width * Height - 1 do
     begin
          Front := Pointer(Integer(pData) + I*3);
          Back := Pointer(Integer(pData) + I*3 + 2);
          Temp := Front^;
          Front^ := Back^;
          Back^ := Temp;
     end;
     fTextureData.Width:=Width;
     fTextureData.Height:=Height;
     fTextureData.ImageData:=pData;
     glGenTextures(1,@fTextureData.TexID);
     glBindTexture(GL_TEXTURE_2D, fTextureData.TexID);
     if not fMIPMapping then
     begin
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,MINFILTER);
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,MAGFILTER);
          glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,fTextureData.Width,fTextureData.Height,0,GL_RGB,GL_UNSIGNED_BYTE,@fTextureData.ImageData[0])
     end
     else
     begin
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,MINFILTER);
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,MAGFILTER);
          gluBuild2DMipmaps(GL_TEXTURE_2D, 3, fTextureData.Width, fTextureData.Height, GL_RGB,GL_UNSIGNED_BYTE, @fTextureData.ImageData[0]);
     end;

     result :=TRUE;
end;

function TTexture.LoadWAL(aFilename,PaletteFilename:string):boolean;
var
   TexFile,PalFile:file;
   header:TWALHeader;
   tempImageData:array of GLubyte;
   i,CurIndex:integer;
   Palette:array[0..255,0..2] of byte;
begin
     result:=false;
     if  not FileExists(aFilename) then exit;
     if not FileExists(PaletteFilename) then exit;

     Assign(TexFile,aFilename);
     Reset(TexFile,1);

     Assign(PalFile,PaletteFilename);
     Reset(PalFile,1);

     BlockRead(TexFile,header,SizeOf(header));
     fTextureData.Width:=header.Width;
     fTextureData.Height:=header.Height;

     SetLength(tempImagedata,header.Width*header.Height);
     SetLength(fTextureData.ImageData,header.Width*header.Height*3);

     BlockRead(TexFile,tempImageData[0],header.Width*header.Height);
     BlockRead(PalFile,Palette,SizeOf(Palette));

     if WAL_Brightness<>0 then
        for CurIndex:=0 to 255  do
        begin
             if Palette[CurIndex,0]+WAL_Brightness<255 then
                Inc(Palette[CurIndex,0],WAL_Brightness);
             if Palette[CurIndex,1]+WAL_Brightness<255 then
                Inc(Palette[CurIndex,1],WAL_Brightness);
             if Palette[CurIndex,2]+WAL_Brightness<255 then
                Inc(Palette[CurIndex,2],WAL_Brightness);
        end;
     i:=0;
     for CurIndex:=0 to header.Width*header.Height do
     begin
          fTextureData.ImageData[i]:=Palette[tempImageData[CurIndex]][0];
          Inc(i);
          fTextureData.ImageData[i]:=Palette[tempImageData[CurIndex]][1];
          Inc(i);
          fTextureData.ImageData[i]:=Palette[tempImageData[CurIndex]][2];
          Inc(i);
     end;
     Close(TexFile);
     Close(PalFile);

     glGenTextures(1,@fTextureData.TexID);
     glBindTexture(GL_TEXTURE_2D, fTextureData.TexID);
     if not fMIPMapping then
     begin
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,MINFILTER);
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,MAGFILTER);
          glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,fTextureData.Width,fTextureData.Height,0,GL_RGB,GL_UNSIGNED_BYTE,fTextureData.ImageData)
     end
     else
     begin
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,MINFILTER);
          glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,MAGFILTER);
          gluBuild2DMipmaps(GL_TEXTURE_2D, 3, fTextureData.Width, fTextureData.Height, GL_RGB,GL_UNSIGNED_BYTE, fTextureData.ImageData);
     end;

     result:=true;
end;
function TTexture.LoadJPEG(aFilename: String):Boolean;
var
  Data : Array of LongWord;
  W, Width : Integer;
  H, Height : Integer;
  BMP : TBitmap;
  JPG : TJPEGImage;
  C : LongWord;
  Line : ^LongWord;
begin
  result :=FALSE;
  JPG:=TJPEGImage.Create;

  try
    JPG.LoadFromFile(aFilename);
  except
    MessageBox(0, PChar('Couldn''t load JPG - "'+ aFilename +'"'), PChar('BMP Unit'), MB_OK);
    Exit;
  end;

  // Create Bitmap
  BMP:=TBitmap.Create;
  BMP.pixelformat:=pf32bit;
  BMP.width:=JPG.width;
  BMP.height:=JPG.height;
  BMP.canvas.draw(0,0,JPG);        // Copy the JPEG onto the Bitmap

  Width :=BMP.Width;
  Height :=BMP.Height;
  SetLength(Data, Width*Height);

  For H:=0 to Height-1 do
  Begin
    Line :=BMP.scanline[Height-H-1];   // flip JPEG
    For W:=0 to Width-1 do
    Begin
      c:=Line^ and $FFFFFF; // Need to do a color swap
      Data[W+(H*Width)] :=(((c and $FF) shl 16)+(c shr 16)+(c and $FF00)) or $FF000000;  // 4 channel.
      inc(Line);
    End;
  End;

  fTextureData.Width:=Width;
  fTextureData.Height:=Height;
  fTextureData.ImageData:=addr(Data[0]);

  glGenTextures(1,@fTextureData.TexID);
  glBindTexture(GL_TEXTURE_2D, fTextureData.TexID);
  if not fMIPMapping then
  begin
       glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,MINFILTER);
       glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,MAGFILTER);
       glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,fTextureData.Width,fTextureData.Height,0,GL_RGBA,GL_UNSIGNED_BYTE,fTextureData.ImageData)
  end
  else
  begin
       glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,MINFILTER);
       glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,MAGFILTER);
       gluBuild2DMipmaps(GL_TEXTURE_2D, 3, fTextureData.Width, fTextureData.Height, GL_RGBA,GL_UNSIGNED_BYTE, fTextureData.ImageData);
  end;

  BMP.free;
  JPG.free;

  result :=TRUE;
end;

function TTexture.LoadTexture(aFilename: string): boolean;
var
   ext:string;
begin
     ext:=UpperCase(ExtractFileExt(aFilename));
     if ext='.JPG' then
     begin
          result:=LoadJPEG(aFilename);
          exit;
     end;
     if ext='.BMP' then
     begin
          result:=LoadBitmap(aFilename);
          exit;
     end;
     if ext='.TGA' then
     begin
          result:=LoadTGA(aFilename);
          exit;
     end;
     if ext='.WAL' then
     begin
          result:=LoadWAL(aFilename,ExtractFilepath(aFilename)+'palette.pal');
          exit;
     end
     else
         result:=false;
end;

end.
