unit Main;

interface

uses
    Windows,
    Messages,
    SysUtils,
    Classes,
    Graphics,
    Controls,
    Forms,
    GL,
    GLU,
    GLUT,
    GLClass,
    BaseTypes,
    GLTexture,
    Dialogs,
    ExtCtrls,
    Camera;

type
  TMainForm = class(TForm)
    Timer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
    //Declare an instance so we can use it
    GL:TOpenGL;

    //The camera
    Camera:TRUVCamera;

    //The texture
    tex:TTexture;

    //Some mouse movement to add spice to this drab tut
    MousePressed:boolean;
    xStart,xDelta,yStart,yDelta:integer;
    xRot,yRot:single;
    procedure RenderCube;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.RenderCube;
begin
      glBegin(GL_QUADS);
        // Front Face
        glNormal3f( 0.0, 0.0, 0.5);
        glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
        glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
        glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0,  1.0);
        glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0,  1.0);
        // Back Face
        glNormal3f( 0.0, 0.0,-0.5);
        glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
        glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
        glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
        glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
        // Top Face
        glNormal3f( 0.0, 0.5, 0.0);
        glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
        glTexCoord2f(0.0, 0.0); glVertex3f(-1.0,  1.0,  1.0);
        glTexCoord2f(1.0, 0.0); glVertex3f( 1.0,  1.0,  1.0);
        glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
        // Bottom Face
        glNormal3f( 0.0,-0.5, 0.0);
        glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, -1.0, -1.0);
        glTexCoord2f(0.0, 1.0); glVertex3f( 1.0, -1.0, -1.0);
        glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
        glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
        // Right face
        glNormal3f( 0.5, 0.0, 0.0);
        glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
        glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
        glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0,  1.0);
        glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
        // Left Face
        glNormal3f(-0.5, 0.0, 0.0);
        glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
        glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
        glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0,  1.0);
        glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
      glEnd();
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
     //Create it. We are going to render to the form.
     GL:=TOpenGL.Create(Self);
     //Check if the user wants to run fullscreen or not.
     GL.FullScreen:=MessageBox(0,'Would you like to run fullscreen?',
                                           'TOpenGL Demo',
                                           MB_YESNO or MB_ICONQUESTION)=ID_YES;
     //That's it. Cool, aint it?
     //Any changes you want to make will never muck up this code.
     GL.InitGL;

     glEnable(GL_DEPTH_TEST);
     glEnable(GL_TEXTURE_2D);
     glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
     glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

     //create the texture
     tex:=TTexture.Create;
     //Load the TGA
     tex.LoadTexture('fan3.JPG');
     //Set up the blend function
     tex.Opaque;

     //Create our camera
     Camera:=TRUVCamera.Create;
     Camera.ViewVolume.FOV:=45;
     Camera.ViewVolume.Width:=Width;
     Camera.ViewVolume.Height:=Height;
     Camera.TargetCamera:=true;
     Camera.Viewpoint:=MakeVector(0,0,-5);

     MousePressed:=false;
     xRot:=0;
     yRot:=0;

     //Let the flipping begin
     Timer.Enabled:=true;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
     //Stop the rendering. We dont want to try and render
     //a destroyed object, right?!!
     Timer.Enabled:=false;

     if assigned(GL) then GL.Destroy;
end;

procedure TMainForm.TimerTimer(Sender: TObject);
begin
     //Clear contents of previous render
     GL.ClearAll;

     Camera.Setview;

     tex.Transparency;
     tex.Enable;
     RenderCube;

     //Flip the screen
     GL.Flip;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     case Key of
          //Now quit, we should...like any good app would :)
          VK_ESCAPE:Close;
     end;
end;

procedure TMainForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
     //Mouse button is being held down
     MousePressed:=true;
     //Note the starting position
     xStart:=X;
     yStart:=Y;
end;

procedure TMainForm.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
     //User released the mouse
     MousePressed:=false;
end;

procedure TMainForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
     //Dragging is happening only if the mouse button is down
     if not MousePressed then exit;
     //Calculate how much the mouse has moved
     xDelta:=xStart-X;
     yDelta:=yStart-Y;

     //Adjust rotation and scale so it is'nt too fast
     xRot:=xRot-yDelta/2;
     yrot:=yRot-xDelta/2;

     //Next time, we start from here
     xStart:=X;
     yStart:=Y;

     //Rotate the camera and reset the rotations
     Camera.RotateAboutR(xRot);
     Camera.RotateAboutU(yRot);
     xRot:=0;
     yRot:=0;
end;

end.
